local MAJOR = "StarLibWidget-1.0" 
local MINOR = 1

assert(LibStub, MAJOR.." requires LibStub") 
local LibWidget = LibStub:NewLibrary(MAJOR, MINOR)
if not LibWidget then return end
local LibProperty = LibStub("StarLibProperty-1.0", true)
assert(LibProperty, MAJOR .. " requires StarLibProperty-1.0")
local LibTimer = LibStub("StarLibTimer-1.0", true)
assert(LibTimer, MAJOR .. " requires StarLibTimer-1.0")
local LibError = LibStub("StarLibError-1.0", true)
assert(LibError, MAJOR .. " requires StarLibError-1.0")
local utils = LibStub("StarLibUtils-1.0", true)
assert(utils, MAJOR .. " requires StarLibUtils-1.0")

local LCDError

local pool = setmetatable({}, {__mode = "k"})

if not LibWidget.__index then
	LibWidget.__index = LibWidget
end

-- Create a new StarLibWidgetText object
-- @name WidgetText.New
-- @usage WidgetText:New(visitor, name, config, row, col, layer, type, environment, errorLevel)
-- @param this The inheritance object
-- @param visitor An StarLibCore-1.0 object
-- @parma name This widget's name
-- @param config This widget's configuration
-- @param row This widget's row
-- @param col This widget's column
-- @param layer This widget's layer
-- @param type Dict of widget types.
-- @param environment The code execution environment.
-- @param errorLevel The error level for this object.
-- @return A new StarLibWidgetText object
function LibWidget:New(this, visitor, name, config, row, col, layer, type, environment, errorLevel) 
	
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
	end
	
	obj.error = LibError:New(MAJOR, errorLevel)
	LCDError = obj.error.Print
	
	obj.visitor = visitor
	obj.name = name
	obj.config = config
	obj.row = row
	obj.col = col
	obj.layer = layer
	obj.type = type
	obj.environment = environment
	obj.errorLevel = errorLevel
	obj.lcd = visitor.lcd
	
	--local pos1 = name.find(":")
	--local pos2 = name.find(":")
	
	--obj.layout_base = name.sub(0, pos1)
	--obj.widget_base = name.sub(pos1 + 1, pos2-pos1-1)
	
	obj.started = false
	obj.errorLevel = errorLevel or 3

	for k, v in pairs(obj) do
		this[k] = v
	end
		
	setmetatable(obj, self)
	
	this.WidgetColor = self.WidgetColor

	return obj	
end


-- @name StarLibWidget:Del
-- @usage StarLibWidget:Del([widget]) or object:Del()
-- @param widget An optional widget object
-- @return Nothing
function LibWidget:Del(widget)
	if widget and type(widget) == "table" then
		pool[widget] = true
	else
		pool[self] = true
	end
	if self.child and self.child["Delold"] then
		self.child["Delold"](self.child, widget)
	end
end
    
-- Create a new color
-- @name StarLibWidget.WidgetColor
-- @usage widget:WidgetColor(section, key, C)
-- @param section The configuration section
-- @param key The key within the section -- basic table look up
-- @param C A fresh table where we'll insert color information
-- @return 0 if the color creation fails for some reason, 1 if everything's ok
function LibWidget:WidgetColor(section, key, C)

    C.R = 0;
    C.G = 0;
    C.B = 0;
    C.A = 0;

    local val = self.visitor:FetchRaw(section, key);

    if (val == nil) then
        return 0;
	end

	if utils:color2RGBA(val, C) <  0 then
		LCDErrorr(("widget '%s': ignoring illegal %s color '%s'"):format(self.name, key, val))
		return 0
	end
	
    return 1;
end

