local MAJOR = "StarLibWidgetBar-1.0" 
local MINOR = 1

assert(LibStub, MAJOR.." requires LibStub") 
local WidgetBar = LibStub:NewLibrary(MAJOR, MINOR)
assert(WidgetBar, "LibStub.NewLibrary failed")
local LibProperty = LibStub("StarLibProperty-1.0", true)
assert(LibProperty, MAJOR .. " requires StarLibProperty-1.0")
local LibTimer = LibStub("StarLibTimer-1.0", true)
assert(LibTimer, MAJOR .. " requires StarLibTimer-1.0")
local LibError = LibStub("StarLibError-1.0", true)
assert(LibError, MAJOR .. " requires StarLibError-1.0")
local LibWidget = LibStub("StarLibWidget-1.0", true)
assert(LibWidget, MAJOR .. " requires StarLibWidget-1.0")

local pool = setmetatable({}, {__mode = "k"})

if not WidgetBar.__index then
	WidgetBar.__index = WidgetBar
end

local LCDError

SCHARS = {
    {31, 31, 31, 31, 31, 31, 31, 31},
    {31, 16, 16, 16, 16, 16, 16, 31},
    {31,  0,  0,  0,  0,  0,  0, 31},
    {31,  1,  1,  1,  1,  1,  1, 31},
    {31, 31, 31, 31,  0,  0,  0,  0},
    { 0,  0,  0,  0, 31, 31, 31, 31}};
	
WidgetBar.defaults = {
}

-- Create a new StarLibWidgetBar object
-- @name WidgetBar.New
-- @usage WidgetBar:New(visitor, name, config, row, col, layer, fontString, env, errorLevel)
-- @param visitor An StarLibCore-1.0 object
-- @param config This widget's settings
-- @param row This widget's row
-- @param col This widget's column
-- @param layer This widget's layer
-- @param fontString The fontString that we'll update, if any
-- @param env The environment for the script
-- @param config The settings. You can provide these fields: string, prefix, postfix, precision, align, update, scroll, speed, direction, cols, offset, str, update, speed. Check the source for further details. Some fields have defaults.
-- @param errorLevel The errorLevel for this object
-- @return A new StarLibWidgetBar object

function WidgetBar:New(visitor, name, config, row, col, layer, fontString, env, errorLevel, callback, data) 

	assert(fontString, "Please provide the marquee with a font string")
	assert(env, "Please provide the marquee with an environment")
	assert(config, "Please provide the marquee with a config")
	assert(config.name, "Please provide the marquee with a name")
	assert(config.string, config.name .. ": Please provide the marquee with a string")
			
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
	end
	
	obj.error = LibError:New(MAJOR, errorLevel)
	LCDError = obj.error.Print
		
	obj.errorLevel = errorLevel or 3

	obj.callback = callback
	obj.data = data

	obj.widget = LibWidget:New(obj, visitor, name, config, row, col, layer, {bar=true, rc=true, special=true}, env, errorLevel)

    obj.lcd_type = v.type

    obj.RealDraw = callback
	obj.data = data

    obj.expression = LibProperty:New(v, section, "expression", 0);
    obj.expression2 = LibProperty:New(v, section, "expression2");
    obj.expr_min = LibProperty:New(v, section, "min");
    obj.expr_max = LibProperty:New(v, section, "max");

	obj.color_valid = {}
	
    obj.color_valid[0] = obj.widget:WidgetColor(section, "barcolor0", self.color[0]);
    obj.color_valid[1] = obj.widget:WidgetColor(section, "barcolor1", self.color[1]);

    obj.fg_valid = obj.widget:WidgetColor(section, "foreground", self.fg_color_);
    obj.bg_valid = obj.widget:WidgetColor(section, "background", self.bg_color_);

    obj.cols_ = v.cfg:CFGFetch(section, "length", 10)

    obj.rows = v.cfg:CFGFetch(section, "height", 1)

    local val = v.cfg:CFGFetchRaw(section, "direction", "E");
    if( val == "E" ) then
        self.direction = DIR_EAST;
    elseif ( val == "W" ) then
        self.direction = DIR_WEST;
    else
        LCDError(("(Widget %s has unknown direction '%s'; Use (E)ast or (W)est. Using (E)."):format(self.name, val), 2);
        direction_ = DIR_EAST;
    end

    self.update = v.cfg:CFGFetch(section, "update", 1000)

    val = v.cfg:CFGFetchRaw(section, "style", "N");
    
    if( val == "H" ) then
        obj.style = STYLE_HOLLOW;
    elseif (val == "N") then
        obj.style = STYLE_NORMAL;
    else 
        LCDError(("Widget %s has unknown style '%s'; known styles are 'N' or 'H'; using 'N'."):foramt(self.name, val))
        obj.style = STYLE_NORMAL;
    end

	obj.val1 = 0;
	obj.val2 = 0;
	obj.min = 0;
	obj.max = 0;

	obj.timer = LibTimer:New(obj.update, true, self.Update, obj, errorLevel)

--    QObject::connect(visitor_->GetWrapper(), SIGNAL(_ResizeLCD(int, int, int, int)),
--        this, SLOT(Resize(int, int, int, int)));

	setmetatable(obj, self)
		
	return obj	
end

-- Delete a StarLibWidgetBar object
-- @name StarLibWidgetBar.Del
-- @usage StarLibWidgetBar:Del([marq])
-- @param marq A StarLibWidgetBar object
function WidgetBar:Del(marq)
	if not marq then
		marq = self
	end
	pool[marq] = true
	self.widget:Del()
end

function WidgetBar:Resize(rows, cols, old_rows, old_cols)
    local xres = self.visitor.lcd.XRES;
    local yres = self.visitor.lcd.YRES
    local y = rows_ * yres / old_rows;
    local x = cols_ * xres / old_cols;
    local r = row_ * yres / old_rows;
    local c = col_ * xres / old_cols;
    self.rows = round(self.visitor.lcd.LROWS * y / yres);
    self.cols = round(self.visitor.lcd.LCOLS * x / xres);
    self.row = round(self.visitor.lcd.LROWS * r / yres);
    self.col = round(self.visitor.lcd.LCOLS * c / xres);
    self:Update();
end

function WidgetBar:SetupChars() 
	local lcd = self.visitor.lcd
    ch_.clear();
--[[    if( style_ == STYLE_HOLLOW and not expression2_->Valid()) do
        for( unsigned int i = 0; i < 4; i++ ) {
            for( unsigned int j = 0; j < lcd->special_chars.size(); j++ ) do
                SpecialChar c(SCHARS[i], 8);
                if( lcd->special_chars[j] == c ) then
                    ch_[i] = j;
                end
            end

        end
        for(int i = 0; i < 6; i++ ) do
            std::map<const int, char>::iterator it = ch_.find(i);
            if(it != ch_.end()) {
                if(lcd->special_chars.size() >= lcd->CHARS ) then
                    LCDError("Can not allot char for widget: %s",name_.c_str());
                    update_ = -1;
                    return;
                end
                lcd->special_chars.push_back(SpecialChar(SCHARS[i], 8));
                ch_[i] = lcd->special_chars.size() - 1;
                lcd->TextSpecialCharChanged(ch_[i]);
            end
        end
    else if (self.style == STYLE_NORMAL ) then
        for( unsigned int j = 0; j < lcd->special_chars.size(); j++ ) do
            if( lcd->special_chars[j] == SpecialChar(SCHARS[0], 8)) then
                ch_[0] = j;
            end
            if( lcd->special_chars[j] == SpecialChar(SCHARS[4], 8)) do
                ch_[1] = j;
            end
            if( lcd->special_chars[j] == SpecialChar(SCHARS[5], 8)) do
                ch_[2] = j;
            end
        end
        std::map<const int, char>::iterator it;

        it = ch_.find(0);
        if(it == ch_.end()) then
            if(lcd->special_chars.size() >= lcd->CHARS ) then
                LCDError("Can not allot char for bar widget: %s",name_.c_str());
                update_ = -1;
                return;
            end
            lcd->special_chars.push_back(SpecialChar(SCHARS[0], 8));
            ch_[0] = lcd->special_chars.size() - 1;
            lcd->TextSpecialCharChanged(ch_[0]);
        end

        it = ch_.find(1);
        if(it == ch_.end() && expression2_->Valid()) then
            if( lcd->special_chars.size() >= lcd->CHARS ) then
                LCDError("Can not allot char for bar widget: %s",name_.c_str());
                update_ = -1;
                return;
            end
            lcd->special_chars.push_back(SpecialChar(SCHARS[4], 8));
            ch_[1] = lcd->special_chars.size() - 1;
            lcd->TextSpecialCharChanged(ch_[1]);
        end

        it = ch_.find(2);
        if(it == ch_.end() && expression2_->Valid()) then
            if( lcd->special_chars.size() >= lcd->CHARS ) then
                LCDError("Can not allot char for bar widget: %s",name_.c_str());
                update_ = -1;
                return;
            end
            lcd->special_chars.push_back(SpecialChar(SCHARS[5], 8));
            ch_[2] = lcd->special_chars.size() - 1;
            lcd->TextSpecialCharChanged(ch_[2]);
        end
        
    else
        LCDError(("%s: Either choose style (H)ollow or have a 2nd expression."):Format( self.widget_base))
    end
	]]
end

function WidgetBar:Start()
    if( update_ < 0 ) then
        return;
	end
    self.timer:Start();
    self:Update();
end

function WidgetBar:Stop()
    self.ch:Clear();
    self.timer:Stop();
end

function WidgetBar:Update()
    self.expression:Eval();
    local val1 = self.expression:P2N();
    local val2 = val1;
    if( self.expression2.is_valid ) then
        self.expression2:Eval();
        val2 = self.expression2:P2N();
    end

	local max, min
    if( self.expr_min.is_valid ) then
        self.expr_min:Eval();
        min = self.expr_min:P2N();
    else
        min = self.min;
        if( val1 < min ) then
            min = val1;
		end
        if( val2 < min ) then
            min = val2;
		end
    end

    if( self.expr_max.is_valid ) then
        self.expr_max:Eval();
        max = self.expr_max:P2N();
    else
        max = max_;
        if( val1 > max ) then
            max = val1;
		end
        if( val2 > max ) then
            max = val2;
		end
    end

    self.min = min;
    self.max = max;

    if( max > min ) then
        self.val1 = (val1 - min ) / (max - min);
        self.val2 = (val2 - min ) / (max - min);
    else
        self.val1 = 0.0;
        self.val2 = 0.0;
    end

    if(self.Draw) then
        self.Draw(this, self.data);
    else
        LCDError("WidgetBar: no draw method");
	end
end

